/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2006 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.analysis;

import jalview.datamodel.*;

import java.util.*;


/**
 * Takes in a vector of sequences and column start and column end
 * and returns a vector of size (end-start+1). Each element of the
 * vector contains a hashtable with the keys being residues and
 * the values being the count of each residue in that column.
 * This class is used extensively in calculating alignment colourschemes
 * that depend on the amount of conservation in each alignment column.
 * @author $author$
 * @version $Revision$
 */
public class AAFrequency
{
    /** Takes in a !!ARRAY!! of sequences and column start and column end
    * and fills given Vector of size (end-start+1). Each element of the
    * vector contains a hashtable with the keys being residues and
    * the values being the count of each residue in that column.
    * This class is used extensively in calculating alignment colourschemes
    * that depend on the amount of conservation in each alignment column. */

   public static final Hashtable [] calculate(Vector sequences, int start, int end)
   {
     SequenceI [] seqs = new SequenceI[sequences.size()];
     int width = 0;
     for(int i=0; i<sequences.size(); i++)
     {
       seqs[i] = (SequenceI) sequences.elementAt(i);
       if(seqs[i].getLength()>width)
         width = seqs[i].getLength();
     }

     Hashtable [] reply = new Hashtable[width];

     calculate(seqs, start, end, reply);

     return reply;
   }

public static final void calculate(SequenceI[] sequences,
                                  int start, int end,
                                  Hashtable [] result)
{
  Hashtable residueHash;
  int maxCount, nongap, i, j, v, jSize = sequences.length;
  String maxResidue;
  char c;
  float percentage;

  int[] values = new int[132];

  for (i = start; i < end; i++)
  {
    residueHash = new Hashtable();
    maxCount = 0;
    maxResidue = "-";
    nongap = 0;
    values = new int[132];

    for (j = 0; j < jSize; j++)
    {
      if (sequences[j].getLength() > i)
      {
        c = sequences[j].getCharAt(i);

        if ('a' <= c && c <= 'z')
        {
          c -= ('a' - 'A');
        }

        if (jalview.util.Comparison.isGap(c))
        {
          c = '-'; // we always use this for gaps in the property vectors
        }
        else
        {
          nongap++;
        }

        values[c]++;

      }
      else
      {
        values['-']++;
      }
    }

    for (v = 'A'; v < 'Z'; v++)
    {
      if (values[v] == 0 || values[v] < maxCount)
        continue;

      if (values[v] > maxCount)
      {
        maxResidue = String.valueOf( (char) v);
      }
      else if (values[v] == maxCount)
      {
        maxResidue += String.valueOf( (char) v);
      }
      maxCount = values[v];
    }


    residueHash.put("maxCount", new Integer(maxCount));
    residueHash.put("maxResidue", maxResidue);

    percentage = ( (float) maxCount * 100) / (float) jSize;
    residueHash.put("pid_gaps", new Float(percentage));

    percentage = ( (float) maxCount * 100) / (float) nongap;
    residueHash.put("pid_nogaps", new Float(percentage));
    result[i] = residueHash;
  }
}
}


